#include <proto.h>

#include "gtscrwindow.h"
#include "gtframe.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtScrWindow class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init scrwindow.
*
\************************************************************************/
GtScrWindow *GtMkScrWindow(char *n,GtManager *p,GtTag *tgs)
{
	GtScrWindow *obj;
	
	if(!(obj=new GtScrWindow()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtScrWindow object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init scrwindow.
*
\************************************************************************/
GtScrWindow *GtMkScrWindow(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtScrWindow *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkScrWindow(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty scrwindow.
*
\************************************************************************/
GtScrWindow::GtScrWindow(void)
{
	mat=NULL;
	vp=NULL;
	hscr=NULL;
	vscr=NULL;
}

/************************************************************************\
*
* Destroy scrwindow.
*
\************************************************************************/
void GtScrWindow::Destroy(void)
{
	if(mat)
		GtDestroy(mat);
	GtWidget::Destroy();
}

/************************************************************************\
*
* Setup scrwindow.
*
\************************************************************************/
sbool GtScrWindow::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	GtFrame *f;
	ulong sz=10;
	long idel=250,del=100;
	ubyte ap=GTAP_TOGETHER;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_SBSZ,GTRT_LONG,&sz);
	app->GetRes(this,GTR_APLACE,GTRT_STRING,&ap);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_ScrBarSize:
				sz=t->d.val;
				break;
			case GT_ArrowPlace:
				ap=t->d.val;
				if(ap>GTAP_LAST_NR)
					ap=GTAP_TOGETHER;
				break;
			case GT_InitialDelay:
				idel=t->d.val;
				break;
			case GT_Delay:
				del=t->d.val;
				break;
		}
	}
	if(!(mat=GtMkMatrix("swmatrix",p,GT_Columns,2,GT_Rows,2,GT_DONE)))
		return(FALSE);
	if(!(f=GtMkFrame("vpframe",mat,GT_FrameType,GTF_IN,GT_DONE)))
		return(FALSE);
	if(!(vp=GtMkViewPort("viewp",f,NULL)))
		return(FALSE);
	vp->AddCB(GT_ValueChange,ViewChange,this);
	if(!(vscr=GtMkScrollBar("vscrbar",mat,
			GT_Orient,GTO_VERT,
			GT_ScrBarSize,sz,
			GT_ArrowPlace,ap,
			GT_InitialDelay,idel,
			GT_Delay,del,
			GT_Start,0,
			GT_End,0,
			GT_ReportDrag,TRUE,
			GT_ReduceInterval,TRUE,
			GT_DONE)))
		return(FALSE);
	vscr->GetScroller()->AddCB(GT_ValueChange,VertChange,this);
	if(!(hscr=GtMkScrollBar("hscrbar",mat,
			GT_Orient,GTO_HORIZ,
			GT_ScrBarSize,sz,
			GT_ArrowPlace,ap,
			GT_InitialDelay,idel,
			GT_Delay,del,
			GT_Start,0,
			GT_End,0,
			GT_ReportDrag,TRUE,
			GT_ReduceInterval,TRUE,
			GT_DONE)))
		return(FALSE);
	hscr->GetScroller()->AddCB(GT_ValueChange,HorizChange,this);
	return(TRUE);
}

/************************************************************************\
*
* Set scrwindow attributes.
*
\************************************************************************/
sbool GtScrWindow::Set(GtTag *tgs)
{
	GtTag *t;
	ulong sz=0;
	long idel=0,del=0;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_ScrBarSize:
				sz=t->d.val;
				break;
			case GT_InitialDelay:
				idel=t->d.val;
				break;
			case GT_Delay:
				del=t->d.val;
				break;
		}
	}
	if(idel || del)
	{
		hscr->SetV(
			(idel ? GT_InitialDelay : GT_IGNORE),idel,
			(del ? GT_Delay : GT_IGNORE),del,
			GT_DONE);
		vscr->SetV(
			(idel ? GT_InitialDelay : GT_IGNORE),idel,
			(del ? GT_Delay : GT_IGNORE),del,
			GT_DONE);
	}
	if(sz)
	{
		hscr->SetV(GT_ScrBarSize,sz,GT_DONE);
		vscr->SetV(GT_ScrBarSize,sz,GT_DONE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Get scrwindow attributes.
*
\************************************************************************/
void GtScrWindow::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_ScrBarSize:
				*ptr=hscr->GetA(GT_ScrBarSize,0);
				break;
			case GT_InitialDelay:
				*ptr=hscr->GetA(GT_InitialDelay,0);
				break;
			case GT_Delay:
				*ptr=hscr->GetA(GT_Delay,0);
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtScrWindow::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtScrWindow")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Callback activate when child of viewport or viewport change.
*
\************************************************************************/
int GtScrWindow::ViewChange(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtViewPortCD *vcd;
	GtScrWindow *sw;
	int xp,yp;
	
	obj=obj;
	vcd=(GtViewPortCD *)cd;
	sw=(GtScrWindow *)ud;
	xp=vcd->cxpos;
	yp=vcd->cypos;
	if(vcd->xsize<vcd->cxsize)
	{
		if(vcd->cxsize+xp<vcd->xsize)
			xp=vcd->xsize-vcd->cxsize;
	}
	else
		xp=0;
	if(vcd->ysize<vcd->cysize)
	{
		if(vcd->cysize+yp<vcd->ysize)
			yp=vcd->ysize-vcd->cysize;
	}
	else
		yp=0;
	if((xp!=vcd->cxpos) || (yp!=vcd->cypos))
		sw->vp->SetV(GT_ChildXPos,xp,GT_ChildYPos,yp,GT_DONE);
	sw->hscr->SetV(
		GT_Value,-xp,
		GT_End,vcd->cxsize,
		GT_Page,vcd->xsize,
		GT_DONE);
	sw->vscr->SetV(
		GT_Value,-yp,
		GT_End,vcd->cysize,
		GT_Page,vcd->ysize,
		GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Change viewport child position to horizontal scroller value.
*
\************************************************************************/
int GtScrWindow::HorizChange(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrollerCD *scd;
	GtScrWindow *sw;
	
	obj=obj;
	scd=(GtScrollerCD *)cd;
	sw=(GtScrWindow *)ud;
	sw->vp->SetV(GT_ChildXPos,-scd->value,GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Change viewport child position to vertical scroller value.
*
\************************************************************************/
int GtScrWindow::VertChange(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrollerCD *scd;
	GtScrWindow *sw;
	
	obj=obj;
	scd=(GtScrollerCD *)cd;
	sw=(GtScrWindow *)ud;
	sw->vp->SetV(GT_ChildYPos,-scd->value,GT_DONE);
	return(GTCB_CONT);
}

